/**
 * 
 */
package com.jbergin.calculator;

/** Represents a simple four function non-algebraic calculator
 * @author jbergin
 *
 */
public class CalculatorModel {
	
	public class PlusOperator implements Operator {

		public void operate() {
			display += savedValue;
		}

	}

	private final Operator none = new NullOperator();
	private Operator previousOperator = none;
	
	
	public class NullOperator implements Operator {

		public void operate() {
			// nothing
		}

	}

	public interface Operator {
		public void operate();
	}


	private final NumericStrategy remember = new RememberStrategy();
	private final NumericStrategy accumulate = new AccumulateStrategy();
	private NumericStrategy currentStrategy = accumulate;
	
	public class RememberStrategy implements NumericStrategy {

		public void operate(int value) {
			savedValue = display;
			display = value;
			currentStrategy = accumulate;
		}
	}

	public class AccumulateStrategy implements NumericStrategy {

		public void operate(int value) {
			display = display * 10 + value;
		}
	}


	private Key three = new NumericKey(3);
	private Key five = new NumericKey(5);

	/** Represents a numeric key of this calculator
	 * @author jbergin
	 *
	 */
	public class NumericKey implements Key {
		
		private int value;
		
		public NumericKey(int value){
			this.value = value;
		}
		
		public void press() {
			currentStrategy.operate(value);
		}

	}

	/** Represents some key of this calculator
	 * @author jbergin
	 *
	 */
	private interface Key {
		public void press();
	}
	
	private int display = 0;
	private int savedValue = 0;
	
	/** Retrieve the current calculator display value
	 * @return the display value
	 */
	public int display() {
		return display;
	}

	public void pressFive() {
		five.press();
	}

	public void pressThree() {
		three.press();
	}

	public void pressPlus() {
		previousOperator.operate();
		previousOperator = new PlusOperator();
		currentStrategy = remember;		
	}

	public void pressEquals() {
		previousOperator.operate();	
		currentStrategy = remember;
		previousOperator = none;
	}

}
